#!/bin/sh
# Run this to generate all the initial makefiles, etc.

rm -rf autom4te.cache
rm -f  aclocal.m4

srcdir=`dirname $0`
PKG_NAME="EVIDENCE"

DIE=0

(autoconf --version) < /dev/null > /dev/null 2>&1 || {
  echo
  echo "**Error**: You must have \`autoconf' installed to."
  echo "Download the appropriate package for your distribution,"
  echo "or get the source tarball at ftp://ftp.gnu.org/pub/gnu/"
  DIE=1
}

(grep "^AM_PROG_LIBTOOL" $srcdir/configure.in >/dev/null) && {
# (libtoolize --version) < /dev/null > /dev/null 2>&1 || {
  LIBTOOLIZE_CHOICES="$LIBTOOLIZE libtoolize glibtoolize"
  for i in $LIBTOOLIZE_CHOICES ; do
    $i --version </dev/null >/dev/null 2>&1 && LIBTOOLIZE=$i && break
  done
  [ "x$LIBTOOLIZE" = "x" ] &&  {
    echo
    echo "**Error**: You must have \`libtool' installed."
    echo "Get ftp://ftp.gnu.org/pub/gnu/libtool-1.2d.tar.gz"
    echo "(or a newer version if it is available)"
    DIE=1
  }
}

(automake --version) < /dev/null > /dev/null 2>&1 || {
  echo
  echo "**Error**: You must have \`automake' installed."
  echo "Get ftp://ftp.gnu.org/pub/gnu/automake-1.7.tar.gz"
  echo "(or a newer version if it is available)"
  DIE=1
  NO_AUTOMAKE=yes
}


# if no automake, don't bother testing for aclocal
test -n "$NO_AUTOMAKE" || (aclocal --version) < /dev/null > /dev/null 2>&1 || {
  echo
  echo "**Error**: Missing \`aclocal'.  The version of \`automake'"
  echo "installed doesn't appear recent enough."
  echo "Get ftp://ftp.gnu.org/pub/gnu/automake-1.7.tar.gz"
  echo "(or a newer version if it is available)"
  DIE=1
}

if test "$DIE" -eq 1; then
  exit 1
fi

if test -z "$*"; then
  echo "**Warning**: I am going to run \`configure' with no arguments."
  echo "If you wish to pass any to it, please specify them on the"
  echo \`$0\'" command line."
  echo
fi



echo -c "

*******************************************************************************
*******************************************************************************
*******************************************************************************

*** EVIDENCE WILL WORK FINE WITHOUT 9/10 OF THE LIBRARIES IT CHECKS FOR! ***

Almost all of the evidence plugins are optional, only very few are
actually required. Where having a least one out of a class of plugins
is required, there is always one that will build without requiring further
libraries.

Don't let the amount of options put you off. In some cases, evidence needs
a certain functionality that can be found in several libraries. configure
will check for all of those, but it needs only one of them -- if that.
SUCH A CHECK IS NOT NECESSARILY A SUGGESTION TO INSTALL THAT LIBRARY;
it is an attempt to make do with what's already there, so you don't
need to install extra libraries just for evidence!

If something isn't found and there is NO fallback, configure will tell
you what functionality you won't get.

The same holds for backends, providers etc. -- if configure won't build
the \"kio-backend\" because it can't find KDE-headers, that's not
something to worry about if you're not using KDE in the first place.  ; )

In other words, the idea is for you to need to install fewer libraries
this way, not more.  Please keep this in mind before going overboard
with installing every library mentioned in the configure-process \"to
get a complete version of evidence\"!

Thanks,
Azundris

*******************************************************************************
*******************************************************************************
*******************************************************************************
"



case $CC in
xlc )
  am_opt=--include-deps;;
esac

for coin in `find $srcdir -name configure.in -print`
do 
  dr=`dirname $coin`
  if test -f $dr/NO-AUTO-GEN; then
    echo skipping $dr -- flagged as no auto-gen
  else
    echo processing $dr
    macrodirs=`sed -n -e 's,AM_ACLOCAL_INCLUDE(\(.*\)),\1,gp' < $coin`
    ( cd $dr
      aclocalinclude="$ACLOCAL_FLAGS"
      for k in $macrodirs; do
  	if test -d $k; then
          aclocalinclude="$aclocalinclude -I $k"
  	##else 
	##  echo "**Warning**: No such directory \`$k'.  Ignored."
        fi
      done
      if grep "^AM_PROG_LIBTOOL" configure.in >/dev/null; then
	echo "Running libtoolize..."
	libtoolize --force --copy
      fi
      echo "Running aclocal $aclocalinclude ..."
      aclocal $aclocalinclude
      if grep "^AM_CONFIG_HEADER" configure.in >/dev/null; then
	echo "Running autoheader..."
	autoheader
      fi
      echo "Running automake --gnu $am_opt ..."
      automake --add-missing --gnu $am_opt
      echo "Running autoconf ..."
      autoconf
    )
  fi
done

#conf_flags="--enable-maintainer-mode --enable-compile-warnings" #--enable-iso-c

if test x$NOCONFIGURE = x; then
  echo Running $srcdir/configure $conf_flags "$@" ...
  $srcdir/configure $conf_flags "$@" \
  && echo Now please type \`make\' to compile $PKG_NAME.
else
  echo Skipping configure process.
fi
